@@ -10,7 +10,6 @@ use tokio::{
 };
 
 use crate::{
-    block_future,
     error::Error,
     node::NullWorker,
     parser,
@@ -26,7 +25,7 @@ pub struct Context {
     address: AddressSet,
     sender: Sender<NodeMessage>,
     rt: Arc<Runtime>,
-    pub(crate) mailbox: Mailbox,
+    mailbox: Mailbox,
 }
 
 impl Context {
@@ -34,16 +33,8 @@ impl Context {
     pub fn runtime(&self) -> Arc<Runtime> {
         self.rt.clone()
     }
-}
-
-impl Drop for Context {
-    fn drop(&mut self) {
-        let addr = self.address.first();
-        trace!("Running Context::drop()");
-
-        if let Err(e) = block_future(self.rt.as_ref(), async { self.stop_worker(addr).await }) {
-            trace!("Error occured during Context::drop(): {}", e);
-        };
+    pub(crate) fn mailbox_mut(&mut self) -> &mut Mailbox {
+        &mut self.mailbox
     }
 }
 
@@ -86,10 +77,10 @@ impl Context {
     /// Create a new context without spawning a full worker
     pub async fn new_context<S: Into<Address>>(&self, addr: S) -> Result<Context> {
         let addr = addr.into();
-        let ctx = NullWorker::new(Arc::clone(&self.rt), &addr, self.sender.clone());
+        let (ctx, tx) = NullWorker::new(Arc::clone(&self.rt), &addr, self.sender.clone());
 
         // Create a small relay and register it with the internal router
-        let sender = relay::build_root::<NullWorker, _>(Arc::clone(&self.rt), &ctx.mailbox);
+        let sender = relay::build_root::<NullWorker, _>(Arc::clone(&self.rt), tx);
         let (msg, mut rx) = NodeMessage::start_worker(addr.into(), sender);
         self.sender
             .send(msg)
@@ -113,6 +104,9 @@ impl Context {
         let address = address.into();
 
         // Check if the address set is available
+        // TODO: There is not much sense of checking for address collisions here, since in
+        // async environment there may be new Workers started between the check and actual adding
+        // of this Worker to the Router map, so check only should happen during Router::start_worker
         let (check_addrs, mut check_rx) = NodeMessage::check_address(address.clone());
         self.sender
             .send(check_addrs)
@@ -122,13 +116,13 @@ impl Context {
 
         // Build the mailbox first
         let (mb_tx, mb_rx) = channel(32);
-        let mb = Mailbox::new(mb_rx, mb_tx);
+        let mb = Mailbox::new(mb_rx);
 
         // Pass it to the context
         let ctx = Context::new(self.rt.clone(), self.sender.clone(), address.clone(), mb);
 
         // Then initialise the worker message relay
-        let sender = relay::build::<NW, NM>(self.rt.as_ref(), worker, ctx);
+        let sender = relay::build::<NW, NM>(self.rt.as_ref(), worker, ctx, mb_tx);
 
         // Send start request to router
         let (msg, mut rx) = NodeMessage::start_worker(address, sender);
@@ -333,11 +327,9 @@ impl Context {
             loop {
                 match self.next_from_mailbox().await {
                     Ok((m, data, addr)) if check(&m) => break Ok((m, data, addr)),
-                    Ok((_, data, addr)) => {
-                        let onward = data.transport().onward_route.clone();
-                        self.mailbox
-                            .requeue(RelayMessage::direct(addr, data, onward))
-                            .await;
+                    Ok((_, data, _)) => {
+                        // Requeue
+                        self.forward(data).await?;
                     }
                     e => break e,
                 }
@@ -396,10 +388,8 @@ impl Context {
             match parser::message(&data.transport().payload).ok() {
                 Some(msg) => break Ok((msg, data, addr)),
                 None => {
-                    let onward = data.transport().onward_route.clone();
-                    self.mailbox
-                        .requeue(RelayMessage::direct(addr, data, onward))
-                        .await;
+                    // Requeue
+                    self.forward(data).await?;
                 }
             }
         }
